@@ -5,10 +5,10 @@ from pkg_resources import DistributionNotFound, get_distribution
 from message_ix_models.util._logging import setup as setup_logging
 from message_ix_models.util.context import Context
 from message_ix_models.util.importlib import MessageDataFinder
-from message_ix_models.util.scenarioinfo import ScenarioInfo
+from message_ix_models.util.scenarioinfo import ScenarioInfo, Spec
 
 # Expose utility classes
-__all__ = ["Context", "ScenarioInfo"]
+__all__ = ["Context", "ScenarioInfo", "Spec"]
 
 try:
     # Version string for reference in other code
